
    /* 
    Copyright Emanouil Atanassov, 2003
    This file is part of the Fast Generator for the Sobol Sequences.

    The Fast Generator for the Sobol Sequences
    is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

     The Fast Generator for the Sobol Sequences 
    is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */



#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <assert.h>
#include <unistd.h>
#include <sys/time.h>

#include "sobol.h"


double msecs() /* milliseconds */
{
struct timeval tv;
/*
struct timezone tz;
tz.tz_minuteswest=-120;
tz.tz_dsttime=0;

gettimeofday(&tv, &tz);
timezone is obsolete
*/
gettimeofday(&tv, 0);
return 0.000001*((long long)tv.tv_sec*1000000+tv.tv_usec);
 
}

double randd(){
        return (double)rand()/(double)RAND_MAX;
}


double intfun(int intnum, int dimen){
  double s;
  double p;
  int j;
  double x;
  s=1.;
  switch(intnum){
  case 1:{
	for(j=0;j<dimen;j++) {
		x=sobolnextcoordr();
		s=s*(x*x*x+0.75);
  	}
	break;
	}
  case 2:{
	for(j=0;j<dimen;j++)
  	{
   		s=s*fabs(4.0*sobolnextcoordr()-2.0);
   	}
	break;
	}
  case 3:{
	 for(j=1;j<dimen+1;j++) {
		s=s*j*cos(j*sobolnextcoordr());
   	}
	break;
	}
  case 4:{
	s=0.;
  	p=-1.;
	for(j=0;j<dimen;j++) {
   		p=-p*sobolnextcoordr();
   		s=s+p;
   	}
  }
}
  	return s;
}
double exact(int intno,int dimen){
double s,p;
int j;
	if (intno<3){
		return 1;
	}else{
		if (intno==3){
			s=1.;
        		for(j=1;j<dimen+1;j++) {
                		s=s*sin(j);
        		}
        		return s;
		}else{
			p=1./pow(2,dimen);
			if (dimen%2==0){
                   		s=(1-p)/3;
        		} else {
                   		s=(1+p)/3;
        		}
        		return s;
		}
	}
}
//static sobol_real points[SOBOL_MAXDIM]  __attribute__ ((aligned (16)));    
int main(int argc,char**argv){
        int scale;
	int intno;
        double m1,m2;
	double val;
        int j;
	sobol_int i, nterms;
        int dimension;
        sobol_int from=0;
	int choice;
        int s;
        double resu;
        sobol_real *result;
        sobol_int scrambles[SOBOL_MAXDIM];
        m1=msecs();
        result=sobolalignmem(SOBOL_MAXDIM);
        printf("Tell me what to do?\n");
        printf("1. Test output, 2. Measure speed, 3. Calculate integrals\n");
	printf("Choice: ");
	scanf("%i",&choice);
	for(i=0;i<SOBOL_MAXDIM;i++)
                scrambles[i]=(sobol_int)(randd()*pow(2,SOBOL_MAXPOW-1));
	printf("\nDimension: ");
        scanf("%i",&dimension);
        printf("\nBegining from: ");
        #ifdef SOBOL_SINGLE
        scanf("%i",&from);
        #else
        scanf("%Li",&from);
        #endif
        printf("\nScaling factor (usually zero):");
        scanf("%i",&scale);
        printf("\nNumber of terms:");
        #ifdef SOBOL_SINGLE
        scanf("%i",&nterms);
        #else
        scanf("%Li",&nterms);
        #endif
	
	switch (choice){
		case 1:{ /* Test */
        		sobolreaddata();
			sobolpreprocess(dimension, scale, from ,scrambles);     
			for (i=0;i<nterms;i++){
				sobolnextpoint(result); 
				for(s=0;s<dimension;s++){
				//s=(int)(dimension*randd());
                        	resu=sobol_test_getpoint((1<<scale)*i+from,s,scrambles[s]);
				//printf("%f\n",resu);
                        	if (result[s]!= resu){
					#ifdef SOBOL_SINGLE  
                                		printf("Error found  %i %i %f %f\n", i,s,resu, (double)result[s]);
					#else
						printf("Error found  %Li %i %.15f %.15f\n", i,s,resu, result[s]);	
					#endif
					abort();
                        	}
				}
	
        		}
			printf("Success!\n");
			break;
		}
		case 2:{ /* Speed */
			m1=msecs();
        		sobolreaddata();
			sobolpreprocess(dimension, scale, from ,scrambles);   
			for (i=0;i<nterms;i++){
				sobolnextpoint(result); 
                        }
			m2=msecs();
			printf("Last coordinate of the last point%lf\n",(double)result[dimension-1]);
			printf("Time in  seconds and "
			"speed in millions of coordinates per second: %.2lf %.1lf\n",
			m2-m1,
			dimension/(m2-m1));
			break;
		}
		case 3:{
			printf("\nIntegral N (1-4): \n");
        		scanf("%i",&intno);
			val=0;
			m1=msecs(); 
			sobolprepare(0,384>dimension?dimension:384);     
			sobolranfunc=randd;
			for (i=0;i<nterms;i++){
                                sobolnexttraj();
				val+=intfun(intno,dimension);
                        }    
			m2=msecs(); 
			printf("In %lf seconds, the integral with exact value "
			"%lf was approximated as %lf ,\n"
			"using %.0lf points of the sequence, with an absolute error of %.3e.\n",
			(m2-m1),exact(intno,dimension), val/(double)nterms,
(double) nterms, fabs(val/(double)nterms - exact(intno,dimension)));
			break;
		}
	}
	return 0;
}
